"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StateStore = void 0;
const js_deep_patch_1 = require("js-deep-patch");
const rxjs_1 = require("rxjs");
class StateStore {
    constructor(initialValue) {
        this.destroy$ = new rxjs_1.Subject();
        this._value$ = new rxjs_1.BehaviorSubject(initialValue);
        this.value$ = this._value$.asObservable().pipe((0, rxjs_1.takeUntil)(this.destroy$));
    }
    get value() {
        return this._value$.value;
    }
    destroy() {
        this.destroy$.next();
    }
    update(value) {
        this._value$.next(value);
    }
    /**
     * Patch an object value
     */
    patch(partialValue) {
        if (!this.isObject(this.value)) {
            throw new Error('[StateStore::patch] value is not an object');
        }
        const patchedValue = (0, js_deep_patch_1.deepPatch)(this.value, partialValue);
        this.update(patchedValue);
        return patchedValue;
    }
    /**
     * Remove an item from an array by id
     */
    removeItemById(id) {
        this.removeItem({ id });
    }
    /**
     * Remove an item from an array
     */
    removeItem(item) {
        if (!Array.isArray(this.value)) {
            throw new Error('[StateStore::removeItem] value is not an array');
        }
        // TODO: this may remove multiple items, could be improved
        const updatedArray = this.value.filter((el) => {
            if (el.id && item['id']) {
                // Remove based on id, if possible
                return el.id !== item['id'];
            }
            else {
                // Otherwise just compare
                return el !== item;
            }
        });
        this.update(updatedArray);
    }
    /**
     * Update an item in an array
     */
    updateItem(item) {
        if (!Array.isArray(this.value)) {
            throw new Error(`[StateStore::updateItem] value is not an array`);
        }
        if (!item['id']) {
            throw new Error(`[StateStore::updateItem] item doesn't have an id`);
        }
        const updatedValues = this.value.map((v) => {
            if (v.id === item['id']) {
                return item;
            }
            else {
                return v;
            }
        });
        this.update(updatedValues);
    }
    /**
     * Add an item to an array
     */
    addItem(item) {
        if (!Array.isArray(this.value)) {
            throw new Error('[StateStore::addItem] value is not an array');
        }
        // Check, whether we already have the same item
        if (item['id']) {
            const sameItem = this.value.find((el) => el.id === item['id']);
            if (sameItem) {
                return; // It's already added
            }
        }
        // Add the new item
        this.update([...this.value, item]);
    }
    isObject(value) {
        return typeof value === 'object' && !Array.isArray(value) && value !== null;
    }
}
exports.StateStore = StateStore;
//# sourceMappingURL=rxjs-state-store.js.map